/* **********************************************************
 * Copyright 2007 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

#ifndef _VMCF_OBJECT_IMPL_H_
#define _VMCF_OBJECT_IMPL_H_

/**
 * @file VMCFObjectImpl.h
 *
 *    This header file defines the set of object servant life-cycle functions,
 *    and setter/getter functions to access CFObject instance variables.
 *
 * @addtogroup VMCFCore
 * @{
 */

#include <time.h>
#include "VMCFCoreImpl.h"
#include "VMCFClass.h"


/** [FullyQualifiedClass]_create function signature. */

typedef
VMCFCore_CFObject
(*VMCFCore_CFObject_CreateFn)(CORBA_string name,
                              VMCFServer server,
                              CORBA_boolean bind,
                              VMCFCore_CFObject parent,
                              CORBA_Environment *env);


/* Declare the CFObject class methods. */
VMCF_EXTERN VMCF_PROTO_ALLOC(VMCFCore_CFObject);
VMCF_EXTERN VMCF_PROTO_FINI(VMCFCore_CFObject);
VMCF_EXTERN VMCF_PROTO_SUBINST(VMCFCore_CFObject);


VMCF_EXTERN CORBA_boolean
VMCFCore_CFObjectImpl_lock(VMCFCore_CFObject self);

VMCF_EXTERN void
VMCFCore_CFObjectImpl_unlock(VMCFCore_CFObject self);

VMCF_EXTERN VMCFServer
VMCFCore_CFObjectImpl_getServer(VMCFCore_CFObject self);

VMCF_EXTERN VMCFCore_CFObject
VMCFCore_CFObjectImpl_getParent(VMCFCore_CFObject self);

VMCF_EXTERN VMCFCore_CFObject
VMCFCore_CFObjectImpl_addChild(VMCFCore_CFObject self,
                               VMCFCore_CFObject child);

VMCF_EXTERN VMCFCore_CFObject
VMCFCore_CFObjectImpl_getChild(VMCFCore_CFObject self,
                               CORBA_string name);

VMCF_EXTERN CORBA_boolean
VMCFCore_CFObjectImpl_removeChild(VMCFCore_CFObject self,
                                  CORBA_string name);

VMCF_EXTERN CORBA_unsigned_long
VMCFCore_CFObjectImpl_getState(VMCFCore_CFObject self);

VMCF_EXTERN CORBA_unsigned_long
VMCFCore_CFObjectImpl_setState(VMCFCore_CFObject self,
                               CORBA_unsigned_long state);

VMCF_EXTERN time_t
VMCFCore_CFObjectImpl_getLastProbeTime(VMCFCore_CFObject self);

/** @} */

#endif /* _VMCF_OBJECT_IMPL_H_*/

